/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;

/**
 * Representation of the end transaction output builder.
 */
public class EndTransactionOutputBuilder extends DefaultOutputBuilderBase {

    private Map<Class<? extends EndTransactionOutput>, EndTransactionOutput>
            augmentation = new HashMap<>();

    /**
     * Creates an end transaction output builder object.
     */
    public EndTransactionOutputBuilder() {
    }

    /**
     * Creates an end transaction output builder object from a RPC result.
     *
     * @param rpc a common RPC result
     */
    public EndTransactionOutputBuilder(CommonRpcResult rpc) {
        resultCode = rpc.getResultCode();
        message = rpc.getMessage();
    }

    /**
     * Creates an end transaction output builder object from
     * the end transaction output.
     *
     * @param base end transaction output object
     */
    public EndTransactionOutputBuilder(EndTransactionOutput base) {
        message = base.getMessage();
        resultCode = base.getResultCode();
        if (base instanceof EndTransactionOutputImpl) {
            EndTransactionOutputImpl impl = (EndTransactionOutputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Adds augmentation by a class type and an end transaction output.
     * If the augmentation is null, remove the augmentation type key
     * from the hash map.
     *
     * @param augmentationType class that extends EndTransactionOutput
     * @param augmentation EndTransactionOutput object
     * @return self
     */
    public EndTransactionOutputBuilder addAugmentation(
            Class<? extends EndTransactionOutput> augmentationType,
            EndTransactionOutput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType class that extends EndTransactionOutput
     * @return self
     */
    public EndTransactionOutputBuilder removeAugmentation(
            Class<? extends EndTransactionOutput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Returns an end transaction output object.
     *
     * @return an end transaction output object
     */
    public EndTransactionOutput build() {
        return new EndTransactionOutputImpl(this);
    }

    /**
     * Representation of the implement of EndTransactionOutput.
     */
    private static final class EndTransactionOutputImpl
            extends DefaultOutputImplBase
            implements EndTransactionOutput {

        private final Map<Class<? extends EndTransactionOutput>,
                EndTransactionOutput> augmentation;

        private EndTransactionOutputImpl(EndTransactionOutputBuilder base) {
            message = base.message;
            resultCode = base.resultCode;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public int hashCode() {
            return Objects.hash(message, resultCode, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            EndTransactionOutputImpl other = (EndTransactionOutputImpl) obj;
            return Objects.equals(message, other.getMessage()) &&
                    Objects.equals(resultCode, other.getResultCode()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("message", message)
                    .add("resultCode", resultCode)
                    .add("augmentation", augmentation).toString();
        }
    }
}