/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;

/**
 * Representation of structure style delete output builder.
 */
public class StructureStyleNemoDeleteOutputBuilder
        extends DefaultOutputBuilderBase {

    Map<Class<? extends StructureStyleNemoDeleteOutput>,
            StructureStyleNemoDeleteOutput> augmentation
            = new HashMap<>();

    /**
     * Constructs structure style delete output builder.
     */
    public StructureStyleNemoDeleteOutputBuilder() {
    }

    /**
     * Constructs structure style delete output builder
     * from a RPC result.
     *
     * @param rpc common RPC result
     */
    public StructureStyleNemoDeleteOutputBuilder(CommonRpcResult rpc) {
        resultCode = rpc.getResultCode();
        message = rpc.getMessage();
    }

    /**
     * Constructs structure style delete output builder from
     * a structure style delete output.
     *
     * @param base structure style delete output object
     */
    public StructureStyleNemoDeleteOutputBuilder(
            StructureStyleNemoDeleteOutput base) {
        message = base.getMessage();
        resultCode = base.getResultCode();
        if (base instanceof StructureStyleNemoDeleteOutputImpl) {
            StructureStyleNemoDeleteOutputImpl impl =
                    (StructureStyleNemoDeleteOutputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Adds augmentation by a class type and a structure style delete
     * output object.If the augmentation is null,remove the augmentation
     * type key from the hash map.
     *
     * @param augmentationType type that extends StructureStyleNemoDeleteOutput
     * @param augmentation structure style delete output object
     * @return self
     */
    public StructureStyleNemoDeleteOutputBuilder addAugmentation(
            Class<? extends StructureStyleNemoDeleteOutput> augmentationType,
            StructureStyleNemoDeleteOutput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType type that extends StructureStyleNemoDeleteOutput
     * @return self
     */
    public StructureStyleNemoDeleteOutputBuilder removeAugmentation(
            Class<? extends StructureStyleNemoDeleteOutput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds structure style delete output object.
     *
     * @return structure style delete output object
     */
    public StructureStyleNemoDeleteOutput build() {
        return new StructureStyleNemoDeleteOutputImpl(this);
    }

    /**
     * Representation of the implement of StructureStyleNemoDeleteOutput.
     */
    private static final class StructureStyleNemoDeleteOutputImpl
            extends DefaultOutputImplBase
            implements StructureStyleNemoDeleteOutput {

        private final Map<Class<? extends StructureStyleNemoDeleteOutput>,
                StructureStyleNemoDeleteOutput> augmentation;

        private StructureStyleNemoDeleteOutputImpl(
                StructureStyleNemoDeleteOutputBuilder base) {
            message = base.message;
            resultCode = base.resultCode;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public int hashCode() {
            return Objects.hash(message, resultCode, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            StructureStyleNemoDeleteOutputImpl other =
                    (StructureStyleNemoDeleteOutputImpl) obj;
            return Objects.equals(message, other.getMessage()) &&
                    Objects.equals(resultCode, other.getResultCode()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("message", message)
                    .add("resultCode", resultCode)
                    .add("augmentation", augmentation).toString();
        }
    }
}