/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent.structure.style.nemo.delete.input;

import org.onosproject.nemo.model.common.ConnectionId;
import org.onosproject.nemo.model.common.FlowId;
import org.onosproject.nemo.model.common.NodeId;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Representation of objects builder.
 */
public class ObjectsBuilder {

    private Map<Class<? extends Objects>, Objects> augmentation = new HashMap<>();

    private List<ConnectionId> connectionIds;

    private List<FlowId> flowIds;

    private List<NodeId> nodeIds;

    /**
     * Constructs an objects builder.
     */
    public ObjectsBuilder() {
    }

    /**
     * Constructs an objects builder from objects.
     *
     * @param base objects
     */
    public ObjectsBuilder(Objects base) {
        connectionIds = base.getConnectionIds();
        flowIds = base.getFlowIds();
        nodeIds = base.getNodeIds();
        if (base instanceof ObjectsImpl) {
            ObjectsImpl impl = (ObjectsImpl) base;
            if (!impl.augmentation.isEmpty()) {
                this.augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the connection ID list for the builder to use.
     *
     * @param value list of connection ID to set
     * @return self
     */
    public ObjectsBuilder connectionIds(List<ConnectionId> value) {
        connectionIds = value;
        return this;
    }

    /**
     * Sets the flow ID list for the builder to use.
     *
     * @param value list of flow ID to set
     * @return self
     */
    public ObjectsBuilder flowIds(List<FlowId> value) {
        flowIds = value;
        return this;
    }

    /**
     * Sets the node ID list for the builder to use.
     *
     * @param value list of node ID to set
     * @return self
     */
    public ObjectsBuilder nodeIds(List<NodeId> value) {
        nodeIds = value;
        return this;
    }

    /**
     * Adds augmentation by a class type and objects.
     * If the augmentation is null, remove the augmentation type
     * key from the hash map.
     *
     * @param augmentationType class extends Objects
     * @param augmentation the objects
     * @return self
     */
    public ObjectsBuilder addAugmentation(
            Class<? extends Objects> augmentationType, Objects augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation with a class type.
     *
     * @param augmentationType a class type extends Objects
     * @return self
     */
    public ObjectsBuilder removeAugmentation(
            Class<? extends Objects> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds objects.
     *
     * @return objects
     */
    public Objects build() {
        return new ObjectsImpl(this);
    }

    /**
     * Representation of the implement of Objects.
     */
    private static final class ObjectsImpl implements Objects {

        private final List<ConnectionId> connectionIds;
        private final List<FlowId> flowIds;
        private final List<NodeId> nodeIds;

        private final Map<Class<? extends Objects>, Objects> augmentation;

        private ObjectsImpl(ObjectsBuilder builder) {
            connectionIds = new ArrayList<>(builder.connectionIds);
            flowIds = new ArrayList<>(builder.flowIds);
            nodeIds = new ArrayList<>(builder.nodeIds);
            augmentation = new HashMap<>(builder.augmentation);
        }

        @Override
        public List<ConnectionId> getConnectionIds() {
            return new ArrayList<>(connectionIds);
        }

        @Override
        public List<FlowId> getFlowIds() {
            return new ArrayList<>(flowIds);
        }

        @Override
        public List<NodeId> getNodeIds() {
            return new ArrayList<>(nodeIds);
        }

        @Override
        public int hashCode() {
            return java.util.Objects.hash(connectionIds, flowIds, nodeIds, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ObjectsImpl o = (ObjectsImpl) obj;
            return java.util.Objects.equals(connectionIds, o.getConnectionIds()) &&
                    java.util.Objects.equals(flowIds, o.getFlowIds()) &&
                    java.util.Objects.equals(nodeIds, o.getNodeIds()) &&
                    java.util.Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("connection", connectionIds)
                    .add("flow", flowIds)
                    .add("node", nodeIds)
                    .add("augmentation", augmentation).toString();
        }
    }
}