/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.match.item.instance.match.item.value;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import com.google.common.collect.Maps;

/**
 * Representation of building a range value.
 */
public class RangeValueBuilder {

    private Map<Class<? extends RangeValue>, RangeValue>
            augmentation = Maps.newHashMap();
    private Long max;
    private Long min;

    /**
     * Creates a default range value builder.
     */
    public RangeValueBuilder() {
    }

    /**
     * Creates an range value builder from a given range value.
     *
     * @param base the range value
     */
    public RangeValueBuilder(RangeValue base) {
        max = base.getMax();
        min = base.getMin();
        RangeValueImpl impl = (RangeValueImpl) base;
        if (!impl.augmentation.isEmpty()) {
            this.augmentation = new HashMap<>(impl.augmentation);
        }
    }

    /**
     * Sets the maximum range value to be used by the builder.
     *
     * @param value range value
     * @return self
     */
    public RangeValueBuilder Max(Long value) {
        max = value;
        return this;
    }

    /**
     * Sets the minimum range value to be used by the builder.
     *
     * @param value range value
     * @return self
     */
    public RangeValueBuilder Min(Long value) {
        min = value;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public RangeValueBuilder addAugmentation(
            Class<? extends RangeValue> augmentationType,
            RangeValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public RangeValueBuilder removeAugmentation(
            Class<? extends RangeValue> augmentationType) {
        this.augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds an range value from this builder instance.
     *
     * @return range value
     */
    public RangeValue build() {
        return new RangeValueImpl(this);
    }

    /**
     * The implementation of range value.
     */
    private static final class RangeValueImpl implements RangeValue {

        private final Long max;
        private final Long min;
        private final Map<Class<? extends RangeValue>, RangeValue>
                augmentation;

        private RangeValueImpl(RangeValueBuilder base) {
            max = base.max;
            min = base.min;
            this.augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public Long getMax() {
            return max;
        }

        @Override
        public Long getMin() {
            return min;
        }

        @Override
        public int hashCode() {
            return Objects.hash(max, max, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            RangeValue other = (RangeValue) obj;
            if (!Objects.equals(max, other.getMax())) {
                return false;
            }
            if (!Objects.equals(min, other.getMin())) {
                return false;
            }
            RangeValueImpl otherImpl = (RangeValueImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("max", max)
                    .add("min", min)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}