/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.property.instance.property.values;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import com.google.common.collect.Maps;

/**
 * The builder of range value.
 */
public class RangeValueBuilder {
    private Map<Class<? extends RangeValue>, RangeValue> augmentation =
            Maps.newHashMap();
    private long max;
    private long min;

    /**
     * Creates a default user role builder.
     */
    public RangeValueBuilder() {
    }

    /**
     * Creates a range value builder from a range value object.
     *
     * @param base the range value object
     */
    public RangeValueBuilder(RangeValue base) {
        max = base.getMax();
        min = base.getMin();
    }

    /**
     * Sets the maximum value to be used by the builder.
     *
     * @param value maximum range value
     * @return self
     */
    public RangeValueBuilder max(long value) {
        max = value;
        return this;
    }

    /**
     * Sets the minimum value to be used by the builder.
     *
     * @param value minimum range value
     * @return self
     */
    public RangeValueBuilder min(long value) {
        min = value;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public RangeValueBuilder addAugmentation(
            Class<? extends RangeValue> augmentationType,
            RangeValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public RangeValueBuilder removeAugmentation(
            Class<? extends RangeValue> augmentationType) {
            augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a range value from this builder instance.
     *
     * @return range value
     */
    public RangeValue build() {
        return new RangeValueImpl(this);
    }

    /**
     * The implementation of range value interface.
     */
    private static final class RangeValueImpl implements RangeValue {

        private final long max;
        private final long min;
        private Map<Class<? extends RangeValue>, RangeValue> augmentation;

        private RangeValueImpl(RangeValueBuilder base) {
            max = base.max;
            min = base.min;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public long getMax() {
            return max;
        }

        @Override
        public long getMin() {
            return min;
        }

        @Override
        public int hashCode() {
            return Objects.hash(max, min, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            RangeValueImpl o = (RangeValueImpl) obj;
            return Objects.equals(max, o.max) &&
                    Objects.equals(min, o.min) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("max", max)
                    .add("min", min)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}