/*
 * Copyright 2017-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.condition.instance
        .condition.segment;

import org.onosproject.nemo.model.operation.action.instance
        .parameter.values.RangeValue;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class ConditionParameterTargetValueBuilder {

    private final Map<Class<? extends ConditionParameterTargetValue>,
            ConditionParameterTargetValue> augmentation = new HashMap<>();
    private long intValue;
    private RangeValue rangeValue;
    private String stringValue;

    /**
     * Creates an instance by default.
     */
    public ConditionParameterTargetValueBuilder() {
    }

    /**
     * Creates a condition parameter target value builder from
     * a given condition parameter target value.
     *
     * @param base condition parameter target value
     */
    public ConditionParameterTargetValueBuilder(
            ConditionParameterTargetValue base) {
        this.intValue = base.getIntValue();
        this.rangeValue = base.getRangeValue();
        this.stringValue = base.getStringValue();
    }


    /**
     * Sets the list of integer value to be used by the builder.
     *
     * @param value of integer value
     * @return self
     */
    public ConditionParameterTargetValueBuilder intValue(long value) {
        this.intValue = value;
        return this;
    }

    /**
     * Sets the range value to be used by the builder.
     *
     * @param value range value
     * @return self
     */
    public ConditionParameterTargetValueBuilder rangeValue(RangeValue value) {
        this.rangeValue = value;
        return this;
    }

    /**
     * Sets the string value to be used by the builder.
     *
     * @param value string value
     * @return self
     */
    public ConditionParameterTargetValueBuilder stringValue(String value) {
        this.stringValue = value;
        return this;
    }

    /**
     * Adds the augmentation to be used by the builder.
     * Remove the augmentation, if the parameter value is null.
     *
     * @param augmentationType augmentation type
     * @param augmentation the augmentation to be used
     * @return self
     */
    public ConditionParameterTargetValueBuilder addAugmentation(
            Class<? extends ConditionParameterTargetValue> augmentationType,
            ConditionParameterTargetValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation to be used by condition
     * parameter target value builder.
     *
     * @param augmentationType remove augmentation
     * @return a parameter value builder
     */
    public ConditionParameterTargetValueBuilder removeAugmentation(
            Class<? extends ConditionParameterTargetValue> augmentationType) {
        this.augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a new condition parameter target value instance.
     *
     * @return the condition parameter target value instance
     */
    public ConditionParameterTargetValue build() {
        return new ConditionParameterTargetValueImpl(this);
    }

    /**
     * The implementation of condition parameter
     * target value to create an instance.
     */
    private static final class ConditionParameterTargetValueImpl
            implements ConditionParameterTargetValue {

        private final long intValue;
        private final RangeValue rangeValue;
        private final String stringValue;
        private Map<Class<? extends ConditionParameterTargetValue>,
                ConditionParameterTargetValue> augmentation;

        private ConditionParameterTargetValueImpl(
                ConditionParameterTargetValueBuilder base) {
            this.intValue = base.intValue;
            this.rangeValue = base.rangeValue;
            this.stringValue = base.stringValue;
            this.augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public long getIntValue() {
            return intValue;
        }

        @Override
        public RangeValue getRangeValue() {
            return rangeValue;
        }

        @Override
        public String getStringValue() {
            return stringValue;
        }

        @Override
        public int hashCode() {
            return Objects.hash(intValue, rangeValue,
                    stringValue, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() == obj.getClass()) {
                return false;
            }
            ConditionParameterTargetValue other =
                    (ConditionParameterTargetValue) obj;
            ConditionParameterTargetValueImpl otherImpl =
                    (ConditionParameterTargetValueImpl) obj;
            return Objects.equals(intValue, other.getIntValue()) &&
                    Objects.equals(rangeValue, other.getRangeValue()) &&
                    Objects.equals(stringValue, other.getStringValue()) &&
                    Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("intValue", intValue)
                    .add("rangeValue", rangeValue)
                    .add("stringValue", stringValue)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }

}
